/*
 *  LittleOS
 *  Copyright (C) 1998 Lacroix Pascal (placr@mygale.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef KERNEL_H
#define KERNEL_H

#ifndef i386
#define i386
#endif

#include <types.h>
#include <gdt.h>

#include <isa.h>

#define ASM __asm__
#define VOLATILE __volatile__
#define INLINE extern __inline__

#define IRQ(x) (0x20 + x)

#define IDMAP (0xD0000000-0xC0000000)

#ifdef DEBUG

#define DELAY { unsigned long l; for (l = 0; l < 1000000; l++) ; }
#define D(x, y...) ({ printk("Debug(%s in %s:%d): ", __FUNCTION__, __FILE__, __LINE__); printk(x "\n", ## y); DELAY; })
#define assert(x) if (!(x)) { printk("Assertion `%s' failed at line %d in %s\n", #x, __LINE__, __FILE__); panic("halt"); }

#else

#define D(x, y...)
#define assert(x)

#endif

#include <pio.h>
#include <seg.h>
#include <flags.h>

extern ulong total_memory;
extern struct i386_gate idt[256];
extern struct i386_descriptor gdt[GDT_SZ];
extern int sound_delay, hz;

/* main.c */
void panic(const char* msg, ...) __attribute__ ((noreturn));


/* console.c */
void clrscr(void);
void gotoxy(int, int);

void sound(int);
void beep(void);

void printk(const char*, ...);
void init_console(void);


/* memory.c */
void init_memory(void);
void *kmalloc(ulong size);
ulong alloc_page(void);


/* asm.asm */
unsigned char get_byte(unsigned short sel, unsigned long addr);
unsigned short get_word(unsigned short sel, unsigned long addr);
unsigned long get_long(unsigned short sel, unsigned long addr);
void put_byte(unsigned short sel, unsigned long addr, unsigned char val);
void put_word(unsigned short sel, unsigned long addr, unsigned short val);
void put_long(unsigned short sel, unsigned long addr, unsigned long val);


/* start.asm */
extern ulong kernel_base, phys_base;


/* pic.c */
void init_pic(ushort, ushort);
void enable_irq(ushort);
void disable_irq(ushort);


/* pit.c */
void init_pit(int);
ulong getticks(void);
#define get_ticks() getticks()

/* traps.c */
void init_traps(void);


/* timer.c */
void init_timer(void);
extern unsigned long ticks;

/* keyboard.c */
void init_keyboard(void);

/* irq.c */
int alloc_irq(int, ulong, char*);
int free_irq(int);
void irq_stats(void);

/* io.c */
int alloc_io(int, int, char *);

#endif
