/*
 *  LittleOS
 *  Copyright (C) 1998 Lacroix Pascal (placr@mygale.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * keyboard.c - keyboard handling
 */

#include <kernel.h>
#include <string.h>

extern struct i386_gate idt[256];
extern void kb_wait(); /* in asm.asm */

byte leds;
void keyboard_handler(void);

static void set_leds(void)
{
	kb_wait();
        outb(0x60, 0xED);
        kb_wait();
        outb(0x60, leds);
}

int probe_keyboard()
{
        /* probe_keyboard does nothing yet! */
	return 1;
}

void init_keyboard(void)
{
	printk("\eRKeyboard:\eN");
	if (alloc_irq(1, (ulong) &keyboard_handler, "keyboard") == 0) {
        	panic("irq1 can not be used for keyboard");
        }
        if (alloc_io(IO_KBD, IO_KBD + IO_KBDSIZE, "keyboard") == 0) {
        	panic("unable to allocate io range for keyboard");
        }

        leds = 2; /* just num lock (bit 2) */
        set_leds();
        enable_irq(1);
        if (!probe_keyboard()) {
        	printk("warning: not found\n");
                /* should we use com port ? */
        }
        else printk("OK\n");
}

#define CAPS 4
#define NUM 2
#define SCROLL 1

extern int pg_fault;

/* this is called by irq1 in irq.asm */
void keyboard_handler(void)
{
	int c = inb(0x60);
        static int show_all = 0;

        if (c == 1) { /* 1 = ESC */
        	show_all = !show_all;
                printk("Show all scancodes: \e15%s\n\eN", show_all?"Yes" : "No");
	}

        if (show_all) printk("scancode: %d\n", c);

        if (c == 58) { leds = leds ^ CAPS; set_leds();/* caps */}
        else
        if (c == 69) { leds = leds ^ NUM; set_leds(); /* num lock */}
        else
        if (c == 70) { leds = leds ^ SCROLL; set_leds(); /* scroll lock */}
        else
        if (c < 0x80 && !show_all) {
		printk("scancode: %d\n", c);
                if (c == 59) {    /* make a page fault */
                      printk("Make a page fault with: put_byte(ZERO_DS, 5, 123)\n");

                      put_byte(ZERO_DS, 0xA0000 + 5, 123);
                }
        }
}


