/*
 *  LittleOS
 *  Copyright (C) 1998 Lacroix Pascal (placr@mygale.org)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * main.c - kernel entry point (in C)
 */

#include <kernel.h>
#include <stdarg.h>

#include "../version.h"

static char panic_buf[250];

void panic(const char *msg, ...)
{
        va_list args;

        va_start(args, msg);
        vsprintf(panic_buf, msg, args);
        va_end(args);

        puts("\n\e15*** Kernel Panic ***\n\eN");
        puts(panic_buf);
        puts("\n");

        cli();
        while (1) ;
}

extern char btext, etext, bdata, edata, bbss, end;

extern short *video;

void kernel_main(void)
{
	int i;
        long l;

        init_console(); /* this must be first */

        printk("\eBLittle\eROS\eN version "VERSION" by Lacroix Pascal\n");
        printk("%lu bytes code, %lu bytes data and %lu bytes BSS\n",
                &etext - &btext, &edata - &bdata, &end - &bbss);
        init_cpu();	/* cpu detection */
        init_gdt();
        init_pic(0x20, 0x28);
        init_pit(200);
        init_idt();
        init_traps();

        init_memory();  /* memory allocation routines */
        init_paging();

        init_timer();
        init_keyboard();

        sti();

        printk("Press F1 to make a page fault!\n");

        printk("Press a key... (ESC to switch show_all)\n");

        while (1)  ;

}

